@@ -35,6 +35,18 @@ To use NArrayMiss class, you need invoking "require 'narray_miss.rb'" in your sc
 
 require 'narray'
 
+class NArray
+  DUMP_SIZE_LIMIT = 20000
+  def self._load(o) to_na(*Marshal::load(o)).ntoh end
+  def _dump(limit)
+    if size <= DUMP_SIZE_LIMIT
+      Marshal::dump([hton.to_s, typecode, *shape])
+    else
+      raise "size of the NArray (#{size}) is too large to dump "+
+            "(limit: #{DUMP_SIZE_LIMIT})"
+    end
+  end
+end
 
 class NArrayMiss
 
@@ -183,11 +195,11 @@ go back to ((<Index>))
     __new__(array,mask)
   end
   def self.to_nam(*arg)
-    if !Numeric===arg[0] && arg[0].class!=Array && arg[0].class!=NArray
+    if !(Numeric===arg[0]) && arg[0].class!=Array && arg[0].class!=NArray
       raise "first argument must be Numeric, NArray or Array"
     end
-    arg[0] = arg[0].dup if !Numeric===arg[0]
-    if arg.length==2 && !Numeric===arg[1] && arg[1].class!=TrueClass && arg[1].class!=FalseClass then
+    arg[0] = arg[0].dup if !(Numeric===arg[0])
+    if arg.length==2 && !(Numeric===arg[1]) && arg[1].class!=TrueClass && arg[1].class!=FalseClass then
       arg[1] = arg[1].dup
     end
     NArrayMiss.to_nam_no_dup(*arg)
@@ -678,6 +690,8 @@ go back to ((<Index>))
 
   def transpose(*arg)
     obj = self.dup
+    shape = arg.collect{|i| obj.shape[i]}
+    obj.reshape!(*shape)
     obj.set_without_validation( @array.transpose(*arg) )
     obj.set_mask(@mask.transpose(*arg))
     obj
@@ -1177,6 +1191,19 @@ go back to ((<Index>))
   end
 
 
+  def _dump(limit)
+    Marshal::dump([@array._dump(nil),@mask._dump(nil)])
+  end
+  def self._load(o)
+    ary, mask = Marshal::load(o)
+    ary = NArray._load(ary)
+    mask = NArray._load(mask)
+    NArrayMiss.to_nam_no_dup(ary,mask)
+  end
+
+
+
+
 #  private
   private
   def pos(n,shape)
@@ -1307,7 +1334,8 @@ module NMMath
   for operator in func1+func2
     eval <<-EOL,nil,__FILE__,__LINE__+1
     def #{operator}(*x)
-      NMMath::#{operator}(self,*x)
+      x = [self]+x if NArrayMiss===self
+      NMMath::#{operator}(*x)
     end
     EOL
   end
